@@ -77,7 +77,13 @@ static int db_free_results_int(db_conn_t *con);
 static sb_arg_t db_args[] =
 {
   SB_OPT("db-driver", "specifies database driver to use "
-         "('help' to get list of available drivers)", NULL, STRING),
+         "('help' to get list of available drivers)",
+#ifdef USE_MYSQL
+         "mysql",
+#else
+         NULL,
+#endif
+         STRING),
   SB_OPT("db-ps-mode", "prepared statements usage mode {auto, disable}", "auto",
          STRING),
   SB_OPT("db-debug", "print database-specific debug information", "off", BOOL),
