@@ -1,5 +1,5 @@
 /* Copyright (C) 2004 MySQL AB
-   Copyright (C) 2004-2017 Alexey Kopytov <akopytov@gmail.com>
+   Copyright (C) 2004-2018 Alexey Kopytov <akopytov@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -134,12 +134,12 @@ sb_test_t        *current_test;
 static sb_barrier_t thread_start_barrier;
 
 /* structures to handle queue of events, needed for tx_rate mode */
+static pthread_mutex_t    queue_mutex;
+static pthread_cond_t     queue_cond;
 static uint64_t           queue_array[MAX_QUEUE_LEN] CK_CC_CACHELINE;
 static ck_ring_buffer_t   queue_ring_buffer[MAX_QUEUE_LEN] CK_CC_CACHELINE;
 static ck_ring_t          queue_ring CK_CC_CACHELINE;
 
-static int queue_is_full CK_CC_CACHELINE;
-
 static int report_thread_created CK_CC_CACHELINE;
 static int checkpoints_thread_created;
 static int eventgen_thread_created;
@@ -220,9 +220,7 @@ static void report_intermediate(void)
     silence intermediate reports at the end of the test
   */
   if (ck_pr_load_uint(&sb_globals.report_interval) == 0)
-  {
     return;
-  }
 
   sb_counters_agg_intermediate(cnt);
   report_get_common_stat(&stat, cnt);
@@ -290,20 +288,20 @@ void sb_report_cumulative(sb_stat_t *stat)
   log_text(LOG_NOTICE, "");
 
   log_text(LOG_NOTICE, "Latency (ms):");
-  log_text(LOG_NOTICE, "         min:                            %10.2f",
+  log_text(LOG_NOTICE, "         min: %39.2f",
            SEC2MS(stat->latency_min));
-  log_text(LOG_NOTICE, "         avg:                            %10.2f",
+  log_text(LOG_NOTICE, "         avg: %39.2f",
            SEC2MS(stat->latency_avg));
-  log_text(LOG_NOTICE, "         max:                            %10.2f",
+  log_text(LOG_NOTICE, "         max: %39.2f",
            SEC2MS(stat->latency_max));
 
   if (sb_globals.percentile > 0)
-    log_text(LOG_NOTICE, "        %3dth percentile:                %10.2f",
+    log_text(LOG_NOTICE, "        %3dth percentile: %27.2f",
              sb_globals.percentile, SEC2MS(stat->latency_pct));
   else
     log_text(LOG_NOTICE, "         percentile stats:               disabled");
 
-  log_text(LOG_NOTICE, "         sum:                            %10.2f",
+  log_text(LOG_NOTICE, "         sum: %39.2f",
            SEC2MS(stat->latency_sum));
   log_text(LOG_NOTICE, "");
 
@@ -654,7 +652,7 @@ void print_run_mode(sb_test_t *test)
     log_text(LOG_NOTICE, "Debug mode enabled.\n");
   
   if (sb_globals.validate)
-    log_text(LOG_NOTICE, "Additional request validation enabled.\n");
+    log_text(LOG_NOTICE, "Validation checks: on.\n");
 
   if (sb_rand_seed)
   {
@@ -689,7 +687,7 @@ bool sb_more_events(int thread_id)
 
   /* Check if we have a time limit */
   if (sb_globals.max_time_ns > 0 &&
-      sb_timer_value(&sb_exec_timer) >= sb_globals.max_time_ns)
+      SB_UNLIKELY(sb_timer_value(&sb_exec_timer) >= sb_globals.max_time_ns))
   {
     log_text(LOG_INFO, "Time limit exceeded, exiting...");
     return false;
@@ -697,8 +695,12 @@ bool sb_more_events(int thread_id)
 
   /* Check if we have a limit on the number of events */
   if (sb_globals.max_events > 0 &&
-      ck_pr_faa_64(&sb_globals.nevents, 1) >= sb_globals.max_events)
+      SB_UNLIKELY(ck_pr_faa_64(&sb_globals.nevents, 1) >=
+                  sb_globals.max_events))
+  {
+    log_text(LOG_INFO, "Event limit exceeded, exiting...");
     return false;
+  }
 
   /* If we are in tx_rate mode, we take events from queue */
   if (sb_globals.tx_rate > 0)
@@ -706,14 +708,23 @@ bool sb_more_events(int thread_id)
     void *ptr = NULL;
 
     while (!ck_ring_dequeue_spmc(&queue_ring, queue_ring_buffer, &ptr) &&
-           !ck_pr_load_int(&queue_is_full))
-      usleep(500000.0 * sb_globals.threads / sb_globals.tx_rate);
-
-    if (ck_pr_load_int(&queue_is_full))
+           !sb_globals.error)
     {
-      log_text(LOG_FATAL, "Event queue is full. Terminating the worker thread");
+      pthread_mutex_lock(&queue_mutex);
+      pthread_cond_wait(&queue_cond, &queue_mutex);
+      pthread_mutex_unlock(&queue_mutex);
 
-      return false;
+      /* Re-check for global error and time limit after waiting */
+
+      if (sb_globals.error)
+        return false;
+
+      if (sb_globals.max_time_ns > 0 &&
+          SB_UNLIKELY(sb_timer_value(&sb_exec_timer) >= sb_globals.max_time_ns))
+      {
+        log_text(LOG_INFO, "Time limit exceeded, exiting...");
+        return false;
+      }
     }
 
     ck_pr_inc_int(&sb_globals.concurrency);
@@ -829,14 +840,15 @@ static void *worker_thread(void *arg)
   return NULL;
 }
 
+/* Generate exponentially distributed number with a given Lambda */
+
+static inline double sb_rand_exp(double lambda)
+{
+  return -lambda * log(1 - sb_rand_uniform_double());
+}
+
 static void *eventgen_thread_proc(void *arg)
 {
-  unsigned long long pause_ns;
-  unsigned long long next_ns;
-  unsigned long long curr_ns;
-  unsigned long long intr_ns;
-  int                i;
-
   (void)arg; /* unused */
 
   sb_tls_thread_id = SB_BACKGROUND_THREAD_ID;
@@ -846,7 +858,12 @@ static void *eventgen_thread_proc(void *arg)
 
   ck_ring_init(&queue_ring, MAX_QUEUE_LEN);
 
-  i = 0;
+  if (pthread_mutex_init(&queue_mutex, NULL) ||
+      pthread_cond_init(&queue_cond, NULL))
+  {
+    sb_barrier_wait(&thread_start_barrier);
+    return NULL;
+  }
 
   log_text(LOG_DEBUG, "Event generating thread started");
 
@@ -856,41 +873,48 @@ static void *eventgen_thread_proc(void *arg)
 
   eventgen_thread_created = 1;
 
-  curr_ns = sb_timer_value(&sb_exec_timer);
-  /* emulate exponential distribution with Lambda = tx_rate */
-  intr_ns = (long) (log(1 - sb_rand_uniform_double()) /
-                    (-(double) sb_globals.tx_rate)*1000000);
-  next_ns = curr_ns + intr_ns*1000;
+  /*
+    Get exponentially distributed time intervals in nanoseconds with Lambda =
+    tx_rate. Alternatively, we can use Lambda = tx_rate / 1e9
+  */
+  const double lambda = 1e9 / sb_globals.tx_rate;
 
-  for (;;)
+  uint64_t curr_ns = sb_timer_value(&sb_exec_timer);
+  uint64_t intr_ns = sb_rand_exp(lambda);
+  uint64_t next_ns = curr_ns + intr_ns;
+
+  for (int i = 0; ; i = (i+1) % MAX_QUEUE_LEN)
   {
     curr_ns = sb_timer_value(&sb_exec_timer);
+    intr_ns = sb_rand_exp(lambda);
+    next_ns += intr_ns;
 
-    /* emulate exponential distribution with Lambda = tx_rate */
-    intr_ns = (long) (log(1 - sb_rand_uniform_double()) /
-                      (-(double)sb_globals.tx_rate)*1000000);
-
-    next_ns = next_ns + intr_ns*1000;
-    if (next_ns > curr_ns)
+    if (sb_globals.max_time_ns > 0 &&
+        SB_UNLIKELY(curr_ns >= sb_globals.max_time_ns))
     {
-      pause_ns = next_ns - curr_ns;
-      usleep(pause_ns / 1000);
+      /* Wake all waiting threads */
+      pthread_cond_broadcast(&queue_cond);
+      return NULL;
     }
 
+    if (next_ns > curr_ns)
+      sb_nanosleep(next_ns - curr_ns);
+
     /* Enqueue a new event */
     queue_array[i] = sb_timer_value(&sb_exec_timer);
     if (ck_ring_enqueue_spmc(&queue_ring, queue_ring_buffer,
-                             &queue_array[i++]) == false)
+                             &queue_array[i]) == false)
     {
-      ck_pr_store_int(&queue_is_full, 1);
+      sb_globals.error = 1;
       log_text(LOG_FATAL,
                "The event queue is full. This means the worker threads are "
                "unable to keep up with the specified event generation rate");
+      pthread_cond_broadcast(&queue_cond);
       return NULL;
     }
 
-    if (i >= MAX_QUEUE_LEN - 1)
-      i = 0;
+    /* Wake up one waiting thread, if there are any */
+    pthread_cond_signal(&queue_cond);
   }
 
   return NULL;
@@ -914,9 +938,9 @@ static void *report_thread_proc(void *arg)
   sb_rand_thread_init();
 
   if (sb_lua_loaded() && sb_lua_report_thread_init())
-  {
     return NULL;
-  }
+
+  pthread_cleanup_push(sb_lua_report_thread_done, NULL);
 
   log_text(LOG_DEBUG, "Reporting thread started");
 
@@ -931,7 +955,7 @@ static void *report_thread_proc(void *arg)
 
   for (;;)
   {
-    usleep(pause_ns / 1000);
+    sb_nanosleep(pause_ns);
 
     report_intermediate();
 
@@ -944,6 +968,8 @@ static void *report_thread_proc(void *arg)
     pause_ns = next_ns - curr_ns;
   }
 
+  pthread_cleanup_pop(1);
+
   return NULL;
 }
 
@@ -951,7 +977,6 @@ static void *report_thread_proc(void *arg)
 
 static void *checkpoints_thread_proc(void *arg)
 {
-  unsigned long long       pause_ns;
   unsigned long long       next_ns;
   unsigned long long       curr_ns;
   unsigned int             i;
@@ -964,9 +989,9 @@ static void *checkpoints_thread_proc(void *arg)
   sb_rand_thread_init();
 
   if (sb_lua_loaded() && sb_lua_report_thread_init())
-  {
     return NULL;
-  }
+
+  pthread_cleanup_push(sb_lua_report_thread_done, NULL);
 
   log_text(LOG_DEBUG, "Checkpoints report thread started");
 
@@ -983,8 +1008,7 @@ static void *checkpoints_thread_proc(void *arg)
     if (next_ns <= curr_ns)
       continue;
 
-    pause_ns = next_ns - curr_ns;
-    usleep(pause_ns / 1000);
+    sb_nanosleep(next_ns - curr_ns);
 
     log_timestamp(LOG_NOTICE, NS2SEC(sb_timer_value(&sb_exec_timer)),
                   "Checkpoint report:");
@@ -992,8 +1016,7 @@ static void *checkpoints_thread_proc(void *arg)
     report_cumulative();
   }
 
-  if (sb_lua_loaded())
-    sb_lua_report_thread_done();
+  pthread_cleanup_pop(1);
 
   return NULL;
 }
@@ -1050,9 +1073,6 @@ static int run_test(sb_test_t *test)
 
   pthread_mutex_init(&sb_globals.exec_mutex, NULL);
 
-
-  queue_is_full = 0;
-
   sb_globals.threads_running = 0;
 
   /* Calculate the required number of threads for the start barrier */
@@ -1151,6 +1171,30 @@ static int run_test(sb_test_t *test)
   if (test->ops.cleanup != NULL && test->ops.cleanup() != 0)
     return 1;
 
+  if (report_thread_created)
+  {
+    if (sb_thread_cancel(report_thread) || sb_thread_join(report_thread, NULL))
+      log_errno(LOG_FATAL, "Terminating the reporting thread failed.");
+  }
+
+  if (eventgen_thread_created)
+  {
+    /*
+      When a time limit is used, the event generation thread may terminate
+      itself.
+    */
+    if ((sb_thread_cancel(eventgen_thread) ||
+         sb_thread_join(eventgen_thread, NULL)) && sb_globals.max_time_ns == 0)
+      log_text(LOG_FATAL, "Terminating the event generator thread failed.");
+  }
+
+  if (checkpoints_thread_created)
+  {
+    if (sb_thread_cancel(checkpoints_thread) ||
+        sb_thread_join(checkpoints_thread, NULL))
+      log_errno(LOG_FATAL, "Terminating the checkpoint thread failed.");
+  }
+
   /* print test-specific stats */
   if (!sb_globals.error)
   {
@@ -1170,27 +1214,6 @@ static int run_test(sb_test_t *test)
   if (test->ops.done != NULL)
     (*(test->ops.done))();
 
-  /* Delay killing the reporting threads to avoid mutex lock leaks */
-  if (report_thread_created)
-  {
-    if (sb_thread_cancel(report_thread) || sb_thread_join(report_thread, NULL))
-      log_errno(LOG_FATAL, "Terminating the reporting thread failed.");
-  }
-
-  if (eventgen_thread_created)
-  {
-    if (sb_thread_cancel(eventgen_thread) ||
-        sb_thread_join(eventgen_thread, NULL))
-      log_text(LOG_FATAL, "Terminating the event generator thread failed.");
-  }
-
-  if (checkpoints_thread_created)
-  {
-    if (sb_thread_cancel(checkpoints_thread) ||
-        sb_thread_join(checkpoints_thread, NULL))
-      log_errno(LOG_FATAL, "Terminating the checkpoint thread failed.");
-  }
-
   return sb_globals.error != 0;
 }
 
